// ----------------------------------------------------------------
// The contents of this file are distributed under the CC0 license.
// See http://creativecommons.org/publicdomain/zero/1.0/
// ----------------------------------------------------------------

#include "Controller.h"
#include "View.h"
#include <iostream>

using std::cout;

Controller::Controller(Model& model, View& view, bool* pKeepRunning)
: m_model(model), m_view(view), m_pKeepRunning(pKeepRunning), m_spaceDown(false)
{
}

Controller::~Controller()
{
}

void Controller::update()
{
	SDL_Event event;
	while(SDL_PollEvent(&event))
	{
		if(event.type == SDL_QUIT)
			*m_pKeepRunning = false;
	}
	const Uint8* keys = SDL_GetKeyboardState(NULL);
	if(keys[SDL_SCANCODE_ESCAPE])
		*m_pKeepRunning = false;
	int mouseX, mouseY;
	Uint32 mouse_buttons = SDL_GetMouseState(&mouseX, &mouseY);
	if(keys[SDL_SCANCODE_SPACE])
	{
		if(!m_spaceDown)
		{
			m_spaceDown = true;
			m_view.visualizing = !m_view.visualizing; // Toggle the display on/off. (Training is much faster when off.)
			if(m_view.visualizing)
				cout << "Visualize mode (slow)\n";
			else
				cout << "No-visualize mode (fast)\n";
			cout.flush();
		}
	}
	else
		m_spaceDown = false;
	m_model.applyForce(0.0);
	if(keys[SDL_SCANCODE_RIGHT])
	{
		m_model.applyForce(0.1);
	}
	if(keys[SDL_SCANCODE_LEFT])
	{
		m_model.applyForce(-0.1);
	}
	if(mouse_buttons & SDL_BUTTON(SDL_BUTTON_LEFT))
	{
		//m_model.onClick(mouseX, mouseY);
	}
}


